@@ -49,7 +49,7 @@
 #include "rb-util.h"
 #include "rb-debug.h"
 
-static GPrivate * private_is_primary_thread;
+static GPrivate private_is_primary_thread;
 
 /**
  * rb_true_function: (skip):
@@ -139,9 +139,9 @@ rb_gvalue_compare (GValue *a, GValue *b)
 			retval = 1;
 		break;
 	case G_TYPE_CHAR:
-		if (g_value_get_char (a) < g_value_get_char (b))
+		if (g_value_get_schar (a) < g_value_get_schar (b))
 			retval = -1;
-		else if (g_value_get_char (a) == g_value_get_char (b))
+		else if (g_value_get_schar (a) == g_value_get_schar (b))
 			retval = 0;
 		else
 			retval = 1;
@@ -409,7 +409,7 @@ gboolean
 rb_is_main_thread (void)
 {
 	if (g_thread_supported()) {
-		return GPOINTER_TO_UINT(g_private_get (private_is_primary_thread)) == 1;
+		return GPOINTER_TO_UINT(g_private_get (&private_is_primary_thread)) == 1;
 	} else {
 		return TRUE;
 	}
@@ -423,19 +423,19 @@ purge_useless_threads (gpointer data)
 }
 
 
-static GStaticRecMutex rb_gdk_mutex;
+static GRecMutex rb_gdk_mutex;
 static gboolean mutex_recurses;
 
 static void
 _threads_enter (void)
 {
-	g_static_rec_mutex_lock (&rb_gdk_mutex);
+	g_rec_mutex_lock (&rb_gdk_mutex);
 }
 
 static void
 _threads_leave (void)
 {
-	g_static_rec_mutex_unlock (&rb_gdk_mutex);
+	g_rec_mutex_unlock (&rb_gdk_mutex);
 }
 
 
@@ -461,23 +461,20 @@ rb_assert_locked (GMutex *mutex)
 void
 rb_threads_init (void)
 {
-	GMutex *m;
+	GMutex m;
 
-	private_is_primary_thread = g_private_new (NULL);
-	g_private_set (private_is_primary_thread, GUINT_TO_POINTER (1));
+	g_private_set (&private_is_primary_thread, GUINT_TO_POINTER (1));
 
-	g_static_rec_mutex_init (&rb_gdk_mutex);
+	g_rec_mutex_init (&rb_gdk_mutex);
 	gdk_threads_set_lock_functions (_threads_enter, _threads_leave);
 	gdk_threads_init ();
 
-	m = g_mutex_new ();
-
-	g_mutex_lock (m);
-	mutex_recurses = g_mutex_trylock (m);
+	g_mutex_init (&m);
+	g_mutex_lock (&m);
+	mutex_recurses = g_mutex_trylock (&m);
 	if (mutex_recurses)
-		g_mutex_unlock (m);
-	g_mutex_unlock (m);
-	g_mutex_free (m);
+		g_mutex_unlock (&m);
+	g_mutex_unlock (&m);
 
 	rb_debug ("GMutex %s recursive", mutex_recurses ? "is" : "isn't");
 
@@ -1089,7 +1086,7 @@ rb_signal_accumulator_value_handled (GSignalInvocationHint *hint,
  * @dummy: user data (unused)
  *
  * A #GSignalAccumulator used to combine all returned values into
- * a #GValueArray.
+ * a #GArray of #GValue instances.
  *
  * Return value: %FALSE to abort signal emission, %TRUE to continue
  */
@@ -1099,35 +1096,31 @@ rb_signal_accumulator_value_array (GSignalInvocationHint *hint,
 				   const GValue *handler_return,
 				   gpointer dummy)
 {
-	GValueArray *a;
-	GValueArray *b;
+	GArray *a;
+	GArray *b;
 	int i;
 
 	if (handler_return == NULL)
 		return TRUE;
 
-	a = NULL;
+	a = g_array_sized_new (FALSE, TRUE, sizeof (GValue), 1);
+	g_array_set_clear_func (a, (GDestroyNotify) g_value_unset);
 	if (G_VALUE_HOLDS_BOXED (return_accu)) {
-		a = g_value_get_boxed (return_accu);
-		if (a != NULL) {
-			a = g_value_array_copy (a);
+		b = g_value_get_boxed (return_accu);
+		if (b != NULL) {
+			g_array_append_vals (a, b->data, b->len);
 		}
 	}
 
-	if (a == NULL) {
-		a = g_value_array_new (1);
-	}
-
 	if (G_VALUE_HOLDS_BOXED (handler_return)) {
 		b = g_value_get_boxed (handler_return);
-		for (i=0; i < b->n_values; i++) {
-			GValue *z = g_value_array_get_nth (b, i);
-			a = g_value_array_append (a, z);
+		for (i=0; i < b->len; i++) {
+			a = g_array_append_val (a, g_array_index (b, GValue, i));
 		}
 	}
 
 	g_value_unset (return_accu);
-	g_value_init (return_accu, G_TYPE_VALUE_ARRAY);
+	g_value_init (return_accu, G_TYPE_ARRAY);
 	g_value_set_boxed (return_accu, a);
 	return TRUE;
 }
@@ -1164,14 +1157,14 @@ rb_signal_accumulator_boolean_or (GSignalInvocationHint *hint,
 
 /**
  * rb_value_array_append_data: (skip):
- * @array: #GValueArray to append to
+ * @array: #GArray to append to
  * @type: #GType of the value being appended
  * @Varargs: value to append
  *
  * Appends a single value to @array, collecting it from @Varargs.
  */
 void
-rb_value_array_append_data (GValueArray *array, GType type, ...)
+rb_value_array_append_data (GArray *array, GType type, ...)
 {
 	GValue val = {0,};
 	va_list va;
@@ -1181,7 +1174,7 @@ rb_value_array_append_data (GValueArray *array, GType type, ...)
 
 	g_value_init (&val, type);
 	G_VALUE_COLLECT (&val, va, 0, &err);
-	g_value_array_append (array, &val);
+	g_array_append_val (array, val);
 	g_value_unset (&val);
 
 	if (err)
