@@ -188,6 +188,8 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_paste = impl_paste;
+	source_class->impl_want_uri = rb_device_source_want_uri;
+	source_class->impl_uri_is_source = rb_device_source_uri_is_source;
 
 	mps_class->impl_get_entries = impl_get_entries;
 	mps_class->impl_get_capacity = impl_get_capacity;
@@ -1050,7 +1052,7 @@ send_offline_plays_notification (RBiPodSource *source)
 static void
 rb_ipod_source_entry_changed_cb (RhythmDB *db,
 				 RhythmDBEntry *entry,
-				 GValueArray *changes,
+				 GArray *changes,
 				 RBiPodSource *source)
 {
 	int i;
@@ -1075,8 +1077,8 @@ rb_ipod_source_entry_changed_cb (RhythmDB *db,
 	 * that's the worst that can happen and that's pretty theoretical,
 	 * I don't think avoiding it is worth the effort.
 	 */
-	for (i = 0; i < changes->n_values; i++) {
-		GValue *v = g_value_array_get_nth (changes, i);
+	for (i = 0; i < changes->len; i++) {
+		GValue *v = &g_array_index (changes, GValue, i);
 		RhythmDBEntryChange *change = g_value_get_boxed (v);
 		switch (change->prop) {
 		case RHYTHMDB_PROP_RATING: {
@@ -1284,7 +1286,7 @@ impl_delete_entries (RBMediaPlayerSource *source, GList *entries, RBMediaPlayerS
 	data->destroy_data = destroy_data;
 	data->files = filenames;
 
-	g_thread_create ((GThreadFunc) delete_thread, data, FALSE, NULL);
+	g_thread_new ("ipod-delete", (GThreadFunc) delete_thread, data);
 }
 
 static RBTrackTransferBatch *
