@@ -59,11 +59,6 @@
 #include "rb-mtp-source.h"
 #include "rb-mtp-thread.h"
 
-#if !GLIB_CHECK_VERSION(2,22,0)
-#define g_mount_unmount_with_operation_finish g_mount_unmount_finish
-#define g_mount_unmount_with_operation(m,f,mo,ca,cb,ud) g_mount_unmount(m,f,ca,cb,ud)
-#endif
-
 static void rb_mtp_source_constructed (GObject *object);
 static void rb_mtp_source_dispose (GObject *object);
 static void rb_mtp_source_finalize (GObject *object);
@@ -988,6 +983,14 @@ mtp_device_open_cb (LIBMTP_mtpdevice_t *device, RBMtpSource *source)
 	rb_mtp_thread_get_track_list (priv->device_thread, (RBMtpTrackListCallback) mtp_tracklist_cb, g_object_ref (source), g_object_unref);
 }
 
+static gboolean
+device_loaded_idle (RBMtpSource *source)
+{
+	g_object_set (source, "load-status", RB_SOURCE_LOAD_STATUS_LOADED, NULL);
+	rb_transfer_target_transfer (RB_TRANSFER_TARGET (source), NULL, FALSE);
+	return FALSE;
+}
+
 static void
 mtp_tracklist_cb (LIBMTP_track_t *tracks, RBMtpSource *source)
 {
@@ -1001,9 +1004,7 @@ mtp_tracklist_cb (LIBMTP_track_t *tracks, RBMtpSource *source)
 	}
 	g_object_unref (db);
 
-	g_object_set (source, "load-status", RB_SOURCE_LOAD_STATUS_LOADED, NULL);
-
-	rb_transfer_target_transfer (RB_TRANSFER_TARGET (source), NULL, FALSE);
+	g_idle_add ((GSourceFunc) device_loaded_idle, source);
 }
 
 static char *
@@ -1204,6 +1205,7 @@ prepare_encoder_sink_cb (RBEncoderFactory *factory,
 	LIBMTP_track_t *track;
 	char **bits;
 	char *extension;
+	char *track_str;
 	LIBMTP_filetype_t filetype;
 	gulong track_id;
 	GDate d;
@@ -1235,10 +1237,21 @@ prepare_encoder_sink_cb (RBEncoderFactory *factory,
 	track->genre = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_GENRE);
 
 	/* build up device filename */
-	track->filename = g_strdup_printf ("%s - %s.%s",
+	if (rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER) > 0) {
+		track_str = g_strdup_printf ("%.2lu.%.2lu ",
+					     rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER),
+					     rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER));
+	} else {
+		track_str = g_strdup_printf ("%.2lu ",
+					     rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER));
+	}
+	
+	track->filename = g_strdup_printf ("%s%s - %s.%s",
+					   track_str,
 					   rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST),
 					   rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE),
 					   extension);
+	g_free (track_str);
 	g_free (extension);
 
 	/* construct folder path: artist/album */
