@@ -897,7 +897,7 @@ episode_activated_cb (RBEntryView *view,
 static void
 podcast_entry_changed_cb (RhythmDB *db,
 			  RhythmDBEntry *entry,
-			  GValueArray *changes,
+			  GArray *changes,
 			  RBPodcastSource *source)
 {
 	RhythmDBEntryType *entry_type;
@@ -909,8 +909,8 @@ podcast_entry_changed_cb (RhythmDB *db,
 		return;
 
 	feed_changed = FALSE;
-	for (i = 0; i < changes->n_values; i++) {
-		GValue *v = g_value_array_get_nth (changes, i);
+	for (i = 0; i < changes->len; i++) {
+		GValue *v = &g_array_index (changes, GValue, i);
 		RhythmDBEntryChange *change = g_value_get_boxed (v);
 
 		if (change->prop == RHYTHMDB_PROP_PLAYBACK_ERROR) {
@@ -1219,6 +1219,19 @@ impl_search (RBSource *asource, RBSourceSearch *search, const char *cur_text, co
 	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
 
+static void
+impl_reset_filters (RBSource *asource)
+{
+	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
+	if (source->priv->search_query != NULL) {
+		rhythmdb_query_free (source->priv->search_query);
+		source->priv->search_query = NULL;
+	}
+	rb_source_toolbar_clear_search_entry (source->priv->toolbar);
+
+	rb_property_view_set_selection (source->priv->feeds, NULL);
+}
+
 static gboolean
 impl_show_popup (RBDisplayPage *page)
 {
@@ -1571,7 +1584,7 @@ impl_constructed (GObject *object)
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	/* set up toolbar */
-	source->priv->toolbar = rb_source_toolbar_new (RB_SOURCE (source), ui_manager);
+	source->priv->toolbar = rb_source_toolbar_new (RB_DISPLAY_PAGE (source), ui_manager);
 	rb_source_toolbar_add_search_entry (source->priv->toolbar, "/PodcastSourceSearchMenu", NULL);
 
 	/* pack the feed and post views into the source */
@@ -1720,6 +1733,7 @@ rb_podcast_source_class_init (RBPodcastSourceClass *klass)
 	source_class->impl_search = impl_search;
 	source_class->impl_song_properties = impl_song_properties;
 	source_class->impl_get_delete_action = impl_get_delete_action;
+	source_class->impl_reset_filters = impl_reset_filters;
 
 	g_object_class_install_property (object_class,
 					 PROP_PODCAST_MANAGER,
