@@ -139,7 +139,7 @@ struct RBPlaylistManagerPrivate
 
 	gint dirty;
 	gint saving;
-	GMutex *saving_mutex;
+	GMutex saving_mutex;
 };
 
 enum
@@ -236,8 +236,8 @@ rb_playlist_manager_shutdown (RBPlaylistManager *mgr)
 {
 	g_return_if_fail (RB_IS_PLAYLIST_MANAGER (mgr));
 
-	g_mutex_lock (mgr->priv->saving_mutex);
-	g_mutex_unlock (mgr->priv->saving_mutex);
+	g_mutex_lock (&mgr->priv->saving_mutex);
+	g_mutex_unlock (&mgr->priv->saving_mutex);
 }
 
 /**
@@ -422,7 +422,7 @@ rb_playlist_manager_load_playlists (RBPlaylistManager *mgr)
 	file = g_strdup (mgr->priv->playlists_file);
 
 	/* block saves until the playlists have loaded */
-	g_mutex_lock (mgr->priv->saving_mutex);
+	g_mutex_lock (&mgr->priv->saving_mutex);
 
 	exists = g_file_test (file, G_FILE_TEST_EXISTS);
 	if (! exists) {
@@ -459,7 +459,7 @@ rb_playlist_manager_load_playlists (RBPlaylistManager *mgr)
 
 	xmlFreeDoc (doc);
 out:
-	g_mutex_unlock (mgr->priv->saving_mutex);
+	g_mutex_unlock (&mgr->priv->saving_mutex);
 	g_free (file);
 }
 
@@ -545,7 +545,7 @@ rb_playlist_manager_save_data (struct RBPlaylistManagerSaveData *data)
 	char *file;
 	char *tmpname;
 
-	g_mutex_lock (data->mgr->priv->saving_mutex);
+	g_mutex_lock (&data->mgr->priv->saving_mutex);
 
 	file = g_strdup (data->mgr->priv->playlists_file);
 	tmpname = g_strconcat (file, ".tmp", NULL);
@@ -562,7 +562,7 @@ rb_playlist_manager_save_data (struct RBPlaylistManagerSaveData *data)
 	g_free (file);
 
 	g_atomic_int_compare_and_exchange (&data->mgr->priv->saving, 1, 0);
-	g_mutex_unlock (data->mgr->priv->saving_mutex);
+	g_mutex_unlock (&data->mgr->priv->saving_mutex);
 
 	g_object_unref (data->mgr);
 
@@ -655,7 +655,7 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
 	if (force)
 		rb_playlist_manager_save_data (data);
 	else
-		g_thread_create ((GThreadFunc) rb_playlist_manager_save_data, data, FALSE, NULL);
+		g_thread_new ("playlist-save", (GThreadFunc) rb_playlist_manager_save_data, data);
 
 	return TRUE;
 }
@@ -845,7 +845,7 @@ rb_playlist_manager_set_automatic_playlist (RBPlaylistManager *mgr,
 					    RBQueryCreator *creator)
 {
 	RhythmDBQueryModelLimitType limit_type;
-	GValueArray *limit_value = NULL;
+	GArray *limit_value = NULL;
 	const char *sort_key;
 	gint sort_direction;
 	GPtrArray *query;
@@ -863,6 +863,9 @@ rb_playlist_manager_set_automatic_playlist (RBPlaylistManager *mgr,
 					   sort_key,
 					   sort_direction);
 	rhythmdb_query_free (query);
+	if (limit_value != NULL) {
+		g_array_unref (limit_value);
+	}
 }
 
 static void
@@ -948,7 +951,7 @@ rb_playlist_manager_cmd_edit_automatic_playlist (GtkAction *action,
 	creator = g_object_get_data (G_OBJECT (playlist), "rhythmbox-playlist-editor");
 	if (creator == NULL) {
 		RhythmDBQueryModelLimitType limit_type;
-		GValueArray *limit_value = NULL;
+		GArray *limit_value = NULL;
 		GPtrArray *query;
 		char *sort_key;
 		gint sort_direction;
@@ -969,7 +972,7 @@ rb_playlist_manager_cmd_edit_automatic_playlist (GtkAction *action,
 									     sort_key,
 									     sort_direction));
 		if (limit_value != NULL) {
-			g_value_array_free (limit_value);
+			g_array_unref (limit_value);
 		}
 		rhythmdb_query_free (query);
 		g_free (sort_key);
@@ -1925,7 +1928,6 @@ rb_playlist_manager_init (RBPlaylistManager *mgr)
 						 RB_TYPE_PLAYLIST_MANAGER,
 						 RBPlaylistManagerPrivate);
 
-	mgr->priv->saving_mutex = g_mutex_new ();
 	mgr->priv->dirty = 0;
 	mgr->priv->saving = 0;
 }
@@ -1986,8 +1988,6 @@ rb_playlist_manager_finalize (GObject *object)
 
 	g_return_if_fail (mgr->priv != NULL);
 
-	g_mutex_free (mgr->priv->saving_mutex);
-
 	g_free (mgr->priv->playlists_file);
 
 	G_OBJECT_CLASS (rb_playlist_manager_parent_class)->finalize (object);
@@ -2096,7 +2096,7 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 			      G_TYPE_NONE,
 			      0, G_TYPE_NONE);
 	/**
-	 * RBPlaylistManager::load-finish
+	 * RBPlaylistManager::load-finish:
 	 * @manager: the #RBPlaylistManager
 	 *
 	 * Emitted when the playlist manager finishes loading the user's
