@@ -226,7 +226,7 @@ rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 
 
 	/**
-	 * RBRemovableMediaManager::create-source-device
+	 * RBRemovableMediaManager::create-source-device:
 	 * @mgr: the #RBRemovableMediaManager
 	 * @device: the device (actually a #GUdevDevice)
 	 *
@@ -247,7 +247,7 @@ rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 			      RB_TYPE_SOURCE,
 			      1, G_TYPE_OBJECT);
 	/**
-	 * RBRemovableMediaManager::create-source-volume
+	 * RBRemovableMediaManager::create-source-volume:
 	 * @mgr: the #RBRemovableMediaManager
 	 * @volume: the #GVolume 
 	 *
@@ -270,7 +270,7 @@ rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 			      1, G_TYPE_VOLUME);
 
 	/**
-	 * RBRemovableMediaManager::create-source-mount
+	 * RBRemovableMediaManager::create-source-mount:
 	 * @mgr: the #RBRemovableMediaManager
 	 * @device_info: a #MPIDDevice containing information on the device
 	 * @mount: the #GMount
@@ -655,13 +655,11 @@ rb_removable_media_manager_add_volume (RBRemovableMediaManager *mgr, GVolume *vo
 
 	mount = g_volume_get_mount (volume);
 	if (mount != NULL) {
-#if GLIB_CHECK_VERSION(2, 20, 0)
 		if (g_mount_is_shadowed (mount) != FALSE) {
 			rb_debug ("mount is shadowed, so ignoring the volume");
 			g_object_unref (mount);
 			return;
 		}
-#endif
 		if (g_hash_table_lookup (priv->mount_mapping, mount) != NULL) {
 			/* this can probably never happen, but it's OK */
 			rb_debug ("already created a source for the mount, so ignoring the volume");
@@ -713,11 +711,9 @@ rb_removable_media_manager_add_mount (RBRemovableMediaManager *mgr, GMount *moun
 	if (g_hash_table_lookup (priv->mount_mapping, mount) != NULL) {
 		return;
 	}
-#if GLIB_CHECK_VERSION(2, 20, 0)
 	if (g_mount_is_shadowed (mount) != FALSE) {
 		return;
 	}
-#endif
 	volume = g_mount_get_volume (mount);
 	if (volume == NULL) {
 		rb_debug ("Unhandled media, no volume for mount");
