@@ -220,8 +220,9 @@ rb_static_playlist_source_create_actions (RBShell *shell)
 
 	g_type_class_unref (klass);
 }
+
 static void
-rb_static_playlist_source_init (RBStaticPlaylistSource *source)
+set_playlist_pixbuf (RBStaticPlaylistSource *source)
 {
 	if (playlist_pixbuf == NULL) {
 		gint size;
@@ -244,6 +245,11 @@ rb_static_playlist_source_init (RBStaticPlaylistSource *source)
 	}
 }
 
+static void
+rb_static_playlist_source_init (RBStaticPlaylistSource *source)
+{
+}
+
 static void
 rb_static_playlist_source_dispose (GObject *object)
 {
@@ -311,6 +317,8 @@ rb_static_playlist_source_constructed (GObject *object)
 	priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
 	psource = RB_PLAYLIST_SOURCE (source);
 
+	set_playlist_pixbuf (source);
+
 	priv->base_model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (psource));
 	g_object_set (priv->base_model, "show-hidden", TRUE, NULL);
 	g_object_ref (priv->base_model);
@@ -351,7 +359,7 @@ rb_static_playlist_source_constructed (GObject *object)
 	gtk_paned_pack2 (GTK_PANED (paned), GTK_WIDGET (songs), TRUE, FALSE);
 
 	/* set up search box / toolbar */
-	priv->toolbar = rb_source_toolbar_new (RB_SOURCE (source), ui_manager);
+	priv->toolbar = rb_source_toolbar_new (RB_DISPLAY_PAGE (source), ui_manager);
 	rb_source_toolbar_add_search_entry (priv->toolbar, "/StaticPlaylistSourceSearchMenu", NULL);
 	g_object_unref (ui_manager);
 
