@@ -85,6 +85,7 @@ static gboolean slider_moved_callback (GtkWidget *widget, GdkEventMotion *event,
 static gboolean slider_release_callback (GtkWidget *widget, GdkEventButton *event, RBHeader *header);
 static void slider_changed_callback (GtkWidget *widget, RBHeader *header);
 static gboolean slider_scroll_callback (GtkWidget *widget, GdkEventScroll *event, RBHeader *header);
+static gboolean slider_focus_out_callback (GtkWidget *widget, GdkEvent *event, RBHeader *header);
 static void time_button_clicked_cb (GtkWidget *button, RBHeader *header);
 
 static void rb_header_elapsed_changed_cb (RBShellPlayer *player, gint64 elapsed, RBHeader *header);
@@ -114,6 +115,7 @@ struct RBHeaderPrivate
 	GtkAdjustment *adjustment;
 	gboolean slider_dragging;
 	gboolean slider_locked;
+	gboolean slider_drag_moved;
 	guint slider_moved_timeout;
 	long latest_set_time;
 
@@ -296,6 +298,10 @@ rb_header_init (RBHeader *header)
 				 "scroll_event",
 				 G_CALLBACK (slider_scroll_callback),
 				 header, 0);
+	g_signal_connect_object (G_OBJECT (header->priv->scale),
+				 "focus-out-event",
+				 G_CALLBACK (slider_focus_out_callback),
+				 header, 0);
 	gtk_scale_set_draw_value (GTK_SCALE (header->priv->scale), FALSE);
 	gtk_widget_set_size_request (header->priv->scale, 150, -1);
 
@@ -322,8 +328,10 @@ rb_header_init (RBHeader *header)
 	/* elapsed time / duration display */
 	header->priv->timelabel = gtk_label_new ("");
 	gtk_widget_set_halign (header->priv->timelabel, GTK_ALIGN_END);
+	gtk_widget_set_no_show_all (header->priv->timelabel, TRUE);
 
 	header->priv->timebutton = gtk_button_new ();
+	gtk_button_set_relief (GTK_BUTTON (header->priv->timebutton), GTK_RELIEF_NONE);
 	gtk_container_add (GTK_CONTAINER (header->priv->timebutton), header->priv->timelabel);
 	g_signal_connect_object (header->priv->timebutton,
 				 "clicked",
@@ -441,6 +449,8 @@ rb_header_playing_song_changed_cb (RBShellPlayer *player, RhythmDBEntry *entry,
 	if (header->priv->entry == entry)
 		return;
 
+	rb_fading_image_start (RB_FADING_IMAGE (header->priv->image), 2000);
+
 	header->priv->entry = entry;
 	if (header->priv->entry) {
 		RBExtDBKey *key;
@@ -463,8 +473,6 @@ rb_header_playing_song_changed_cb (RBShellPlayer *player, RhythmDBEntry *entry,
 
 	g_free (header->priv->image_path);
 	header->priv->image_path = NULL;
-
-	rb_fading_image_start (RB_FADING_IMAGE (header->priv->image), 2000);
 }
 
 static GtkSizeRequestMode
@@ -548,7 +556,11 @@ rb_header_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 
 	/* time button gets its minimum size */
 	gtk_widget_get_preferred_width (RB_HEADER (widget)->priv->songbox, NULL, &info_width);
-	gtk_widget_get_preferred_width (RB_HEADER (widget)->priv->timebutton, &time_width, NULL);
+	if (gtk_widget_get_visible (RB_HEADER (widget)->priv->timelabel)) {
+		gtk_widget_get_preferred_width (RB_HEADER (widget)->priv->timebutton, &time_width, NULL);
+	} else {
+		time_width = 0;
+	}
 
 	info_width = allocation->width - (scale_width + time_width) - (2 * spacing);
 
@@ -569,7 +581,9 @@ rb_header_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 		info_width = 0;
 	}
 
-	if (info_width + scale_width + (2 * spacing) + time_width > allocation->width) {
+	if (time_width == 0) {
+		gtk_widget_hide (RB_HEADER (widget)->priv->timebutton);
+	} else if (info_width + scale_width + (2 * spacing) + time_width > allocation->width) {
 		gtk_widget_hide (RB_HEADER (widget)->priv->timebutton);
 	} else {
 		if (rtl) {
@@ -872,15 +886,18 @@ slider_press_callback (GtkWidget *widget,
 		       RBHeader *header)
 {
 	header->priv->slider_dragging = TRUE;
+	header->priv->slider_drag_moved = FALSE;
 	header->priv->latest_set_time = -1;
 	g_object_notify (G_OBJECT (header), "slider-dragging");
 
+#if !GTK_CHECK_VERSION(3,5,0)
 	/* HACK: we want the behaviour you get with the middle button, so we
 	 * mangle the event.  clicking with other buttons moves the slider in
 	 * step increments, clicking with the middle button moves the slider to
 	 * the location of the click.
 	 */
 	event->button = 2;
+#endif
 
 
 	return FALSE;
@@ -893,6 +910,7 @@ slider_moved_timeout (RBHeader *header)
 
 	apply_slider_position (header);
 	header->priv->slider_moved_timeout = 0;
+	header->priv->slider_drag_moved = FALSE;
 
 	GDK_THREADS_LEAVE ();
 
@@ -910,6 +928,7 @@ slider_moved_callback (GtkWidget *widget,
 		rb_debug ("slider is not dragging");
 		return FALSE;
 	}
+	header->priv->slider_drag_moved = TRUE;
 
 	progress = gtk_adjustment_get_value (header->priv->adjustment);
 	header->priv->elapsed_time = (gint64) ((progress+0.5) * RB_PLAYER_SECOND);
@@ -948,8 +967,10 @@ slider_release_callback (GtkWidget *widget,
 			 GdkEventButton *event,
 			 RBHeader *header)
 {
+#if !GTK_CHECK_VERSION(3,5,0)
 	/* HACK: see slider_press_callback */
 	event->button = 2;
+#endif
 
 	if (header->priv->slider_dragging == FALSE) {
 		rb_debug ("slider is not dragging");
@@ -961,8 +982,11 @@ slider_release_callback (GtkWidget *widget,
 		header->priv->slider_moved_timeout = 0;
 	}
 
-	apply_slider_position (header);
+	if (header->priv->slider_drag_moved)
+		apply_slider_position (header);
+
 	header->priv->slider_dragging = FALSE;
+	header->priv->slider_drag_moved = FALSE;
 	g_object_notify (G_OBJECT (header), "slider-dragging");
 	return FALSE;
 }
@@ -978,6 +1002,8 @@ slider_changed_callback (GtkWidget *widget,
 	if (header->priv->slider_dragging == FALSE &&
 	    header->priv->slider_locked == FALSE) {
 		apply_slider_position (header);
+	} else if (header->priv->slider_dragging) {
+		header->priv->slider_drag_moved = TRUE;
 	}
 }
 
@@ -1006,6 +1032,20 @@ slider_scroll_callback (GtkWidget *widget, GdkEventScroll *event, RBHeader *head
 	return retval;
 }
 
+static gboolean
+slider_focus_out_callback (GtkWidget *widget, GdkEvent *event, RBHeader *header)
+{
+	if (header->priv->slider_dragging) {
+		if (header->priv->slider_drag_moved)
+			apply_slider_position (header);
+
+		header->priv->slider_dragging = FALSE;
+		header->priv->slider_drag_moved = FALSE;
+		g_object_notify (G_OBJECT (header), "slider-dragging");
+	}
+	return FALSE;
+}
+
 static void
 rb_header_update_elapsed (RBHeader *header)
 {
@@ -1016,8 +1056,10 @@ rb_header_update_elapsed (RBHeader *header)
 
 	if (header->priv->entry == NULL) {
 		gtk_label_set_text (GTK_LABEL (header->priv->timelabel), "");
+		gtk_widget_hide (header->priv->timelabel);
 		return;
 	}
+	gtk_widget_show (header->priv->timelabel);
 
 	seconds = header->priv->elapsed_time / RB_PLAYER_SECOND;
 	if (header->priv->duration == 0) {
@@ -1036,6 +1078,7 @@ rb_header_update_elapsed (RBHeader *header)
 
 		/* Translators: remaining time / total time */
 		label = g_strdup_printf (_("-%s / %s"), elapsed, duration);
+		gtk_widget_show (header->priv->timebutton);
 		gtk_label_set_text (GTK_LABEL (header->priv->timelabel), label);
 
 		g_free (elapsed);
